@@ -214,10 +214,12 @@ colourisation_up_to_date(M) :->
 colourise_buffer(M) :->
     "Colour buffer using syntax-rules"::
     send(M, remove_syntax_fragments),
-    get(M, text_buffer, TB),
-    get(TB, generation, Generation),
-    send(TB, coloured_generation, Generation).
+    send(M, update_bookmarks).
 
+update_bookmarks(M) :->
+    "Update location and contents of bookmarks"::
+    send(M, for_all_fragments,
+         if(message(@arg1, send_hyper, bookmark, update))).
 
 remove_syntax_fragments(M,
                         From:from=[int], To:to=[int],
@@ -670,13 +672,15 @@ replace_end(M) :->
         fail
     ).
 
-
 bookmark_line(M) :->
     "Create a mark in the mark-list"::
-    get(M, caret, Caret),
-    get(M, scan, Caret, line, 0, start, SOL),
-    send(@emacs_mark_list, append_hit, M?text_buffer, SOL).
-
+    (   get(M, selection, point(Start, End))
+    ->  true
+    ;   get(M, caret, Caret),
+        get(M, scan, Caret, line, 0, start, Start),
+        get(M, scan, Start, line, 0, end, End)
+    ),
+    send(@emacs_mark_list, append_hit, M?text_buffer, Start, End).
 
 show_bookmarks(_) :->
     "Show PceEmacs bookmarks"::
